#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2014, 2017
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with
#   IBM Corp.
#
#------------------------------------------------------------------------
#  mambo_init.tcl
#
#  This file is called from every .mambo.tcl or .gmambo.tcl file.
#  It provides bootstrapping functions to access any other tcl function
#  automatically.
#------------------------------------------------------------------------

namespace eval MamboInit {
    namespace export *
}


#------------------------------------------------------------------------
#  MamboInit::bootstrap
#       This function will check for the needed tclIndex files, and
#       update them if they are not present or need updating. It will
#       then set up the special tcl variable 'auto_path' which is used
#       to locate and source in functions as they are needed.
#------------------------------------------------------------------------
proc MamboInit::bootstrap {} {

    global auto_path env gvars auto_index
    variable initialized

    if { $initialized } then {
        return
    }

    set gvars(tcl_common_dir) $env(LIB_DIR)/common

    add_tcl_path $env(LIB_DIR)
    add_tcl_path $env(LIB_DIR)/common
    add_tcl_path $env(LIB_DIR)/ppc
    add_tcl_path $env(LIB_DIR)/perf

    register_help_command "help"                    "Displays this help information"
    #register_help_command "make_tclindex"           "Updates the tclIndex files"
    #register_help_command "register_help_command"   "Adds a help command to this help information"

    if { ![ info exists env(EXEC_DIR) ] } then {
        set env(EXEC_DIR)       [ file dirname [ info nameofexecutable ] ]
        set env(PROFILER_DIR)   $env(EXEC_DIR)/lib/ppc/profiler
    }

    # Look for and load in any help_autoload() functions
    init_autohelp

    set initialized 1

}

#------------------------------------------------------------------------
#  MamboInit::register_help_command
#     Add a command to the Tcl Procedure 'help' list.
#------------------------------------------------------------------------
proc MamboInit::register_help_command { cmd desc } {

    variable helpinfo

    set helpinfo($cmd) "$desc"

    return ""
}

#------------------------------------------------------------------------
#  MamboInit::init_autohelp
#     # Look for and load in any help_autoload() functions
#------------------------------------------------------------------------
proc MamboInit::init_autohelp {} {

    global auto_index errorInfo

    # Call a bogus function to update auto_index
    set save_errorInfo $errorInfo
    catch { ____BAD_FUNCTION_NAME___ }
    set errorInfo $save_errorInfo

    foreach help_func [ array names auto_index *::help_autoload ] {
        $help_func
    }
}


#------------------------------------------------------------------------
#  MamboInit::make_tclindex
#     This function should be called by developers when they add new
#     tcl procedures or files.
#------------------------------------------------------------------------
proc MamboInit::make_tclindex {} {

    global gvars

    puts "\tUpdating tclIndex files..."
    foreach tcl_path $gvars(tcl_paths) {
        set index_file [ file join $tcl_path tclIndex ]
        if { [ catch { auto_mkindex $tcl_path } errmsg ] } then {
            puts "\tFailed to create $tcl_path/tclIndex"
            puts "\tReason($errmsg)"
        } else {
            TclUtils::sort_tclIndex $index_file $index_file
            puts "\tCreated $tcl_path/tclIndex"
        }
    }
}

#------------------------------------------------------------------------
#  MamboInit::add_tcl_path
#     Add a directory to the tcl path and update auto_path.
#------------------------------------------------------------------------
proc MamboInit::add_tcl_path { path } {

    global gvars auto_path

    lappend gvars(tcl_paths) $path
    set auto_path [ linsert $auto_path 0 $path ]
}

#------------------------------------------------------------------------
#  MamboInit::help
#     Override mambo built-in help with more complete help
#------------------------------------------------------------------------
proc MamboInit::help {} {

    variable helpinfo

    puts "\nMambo internal commands"
    puts "======================="
    puts [ mambo_help ]

    set sims [display machines]
    if {$sims != {}} {
       puts "\nSimulators"
       puts   "=========="
       foreach sim $sims {
           puts "\t $sim"
       }
    }

    puts "\nTcl procedures"
    puts "=============="

    foreach cmd [ lsort [ array names helpinfo ] ] {
        puts "\t[ format %-30s $cmd ] - $helpinfo($cmd)"
    }

    puts "\nUse 'help' or 'helprecursive' after a command for more help."
    puts "Command name abbreviations are allowed if unambiguous.\n"

}

#------------------------------------------------------------------------
#  MamboInit::create_sim
#     Create a simulator, apply fixation state hooks and return the reference name
#------------------------------------------------------------------------
proc MamboInit::create_sim { model {ref mysim} } {

	define dup $model myconf
    set rtnref [ lindex [ define machine myconf $ref ] 0 ]
	myconf quit
    return $rtnref
}

# bootstrap the system
if { ![ info exists ::MamboInit::initialized ] } then {
    set MamboInit::initialized 0
    rename ::help ::MamboInit::mambo_help
    namespace import -force MamboInit::*
    bootstrap
}

